package net.greenfieldtech.cloudonixsdk.cloudonixdialer.activities;

import android.Manifest;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.support.annotation.NonNull;
import android.support.v4.app.ActivityCompat;
import android.support.v4.view.ViewPager;
import android.util.Log;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.widget.RelativeLayout;
import android.widget.TabHost;
import butterknife.Bind;
import net.greenfieldtech.cloudonixsdk.api.interfaces.IVoIPObserver;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.R;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.adapters.TabsAdapter;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.fragments.ContactsFragment;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.fragments.KeypadFragment;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.fragments.RecentCallsFragment;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.fragments.StatusFragment;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils.SdkFacade;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils.VoIPClient;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.views.TabIndicator;

/**
 * Created by alex on 20.04.16.
 */

public class MainActivity extends BaseActivity {

  private static final String TAG = "MainActivity";

  @Bind(R.id.root)
  RelativeLayout root;
  @Bind(R.id.pager)
  ViewPager viewPager;
  @Bind(android.R.id.tabhost)
  TabHost tabHost;

  private String mRingingCallId = null;

  private TabsAdapter tabsAdapter;

  @Override
  protected int getLayoutResource() {
    return R.layout.activity_main;
  }

  @Override
  protected void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);

    VoIPClient.initWithContext(getApplicationContext());

//        VoIPClient.getInstance().setAccount("","","","",false,443,"");

    setTitleRes(R.string.main_toolbar_title);

    // TODO need to remove it
    setDefaultPreferences();

    if (checkCallingOrSelfPermission(Manifest.permission.READ_CONTACTS)
        != PackageManager.PERMISSION_GRANTED) {
      ActivityCompat.requestPermissions(this, new String[]{Manifest.permission.READ_CONTACTS}, 100);
    } else {
      setupViews();
    }

    if (SdkFacade.findActiveCall(null) != null) {
      String key = SdkFacade.findActiveCall(null);
      String number = VoIPClient.getInstance().getCallUrl(key);

      Intent startCallActivity = new Intent(this, CallActivity.class);
      startCallActivity.putExtra("call_id", key);
      startCallActivity.putExtra("call_number", number);
      startActivity(startCallActivity);
    } else {
      if (VoIPClient.getInstance().isRegistered()
          && VoIPClient.getInstance().getCalls() != null
          && VoIPClient.getInstance().getCalls().length != 0) {
        // TODO check multiple calls
        String callId = VoIPClient.getInstance().getCalls()[0].key;
        String callNumber = VoIPClient.getInstance().getCalls()[0].url;

        Intent startCallActivity = new Intent(this, OutgoingCallActivity.class);
        startCallActivity.putExtra("call_id", callId);
        startCallActivity.putExtra("call_number", callNumber);
        startActivity(startCallActivity);
      }
    }
  }

  private void setupViews() {
    if (tabsAdapter == null) {
      tabHost.setup();
      tabsAdapter = new TabsAdapter(this, tabHost, viewPager);
      initTabs(tabHost);
      viewPager.setCurrentItem(tabsAdapter.getCount() - 1);
      tabsAdapter.setOnTabChangeListener(onTabChangeListener);
    }
  }

  private void initTabs(TabHost mTabHost) {
    TabIndicator status = new TabIndicator(this);
    status.setImage(R.drawable.tab_status_selector);
    tabsAdapter
        .addTab(mTabHost.newTabSpec("Status").setIndicator(status), StatusFragment.class, null);

    TabIndicator buyData = new TabIndicator(this);
    buyData.setImage(R.drawable.tab_recent_calls_selector);
    tabsAdapter
        .addTab(mTabHost.newTabSpec("RecentCalls").setIndicator(buyData), RecentCallsFragment.class,
            null);

    TabIndicator save = new TabIndicator(this);
    save.setImage(R.drawable.tab_contacts_selector);
    tabsAdapter
        .addTab(mTabHost.newTabSpec("Contacts").setIndicator(save), ContactsFragment.class, null);

    TabIndicator sponsoredContent = new TabIndicator(this);
    sponsoredContent.setImage(R.drawable.tab_keypad_selector);
    tabsAdapter
        .addTab(mTabHost.newTabSpec("Keypad").setIndicator(sponsoredContent), KeypadFragment.class,
            null);
  }

  @Override
  protected void onResume() {
    super.onResume();
    VoIPClient.getInstance().addCallbacksListener(mCallbacksListener);
  }

  @Override
  protected void onPause() {
    super.onPause();
    VoIPClient.getInstance().removeCallbacksListener(mCallbacksListener);
  }

  @Override
  public void onBackPressed() {
    int keypadPage = tabsAdapter.getCount() - 1;

    if (viewPager.getCurrentItem() != keypadPage) {
      viewPager.setCurrentItem(keypadPage);
    } else {
      super.onBackPressed();
    }
  }

  @Override
  public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions,
      @NonNull int[] grantResults) {
    super.onRequestPermissionsResult(requestCode, permissions, grantResults);

    for (int i = 0; i < grantResults.length; i++) {
      if (grantResults[i] != PackageManager.PERMISSION_GRANTED) {
        return;
      }
    }

    setupViews();
  }

  private void setDefaultPreferences() {
//        StorageAndPrefsWrapper prefs = StorageAndPrefsWrapper.getInstance();
//        if (!prefs.getBoolean(StorageAndPrefsWrapper.PreferencesKeys.WAS_PREFERENCE_SET, false)) {
    PreferenceManager.setDefaultValues(this, R.xml.pref_codecs_settings, true);
    PreferenceManager.setDefaultValues(this, R.xml.pref_general, true);
    PreferenceManager.setDefaultValues(this, R.xml.pref_transport_types, true);
    PreferenceManager.setDefaultValues(this, R.xml.pref_turn_settings, true);
//            prefs.putBoolean(StorageAndPrefsWrapper.PreferencesKeys.WAS_PREFERENCE_SET, true);
//        }
  }

  private TabsAdapter.OnTabChangeListener onTabChangeListener = new TabsAdapter.OnTabChangeListener() {
    @Override
    public void onTabChanged(int position) {
      View view = MainActivity.this.getCurrentFocus();
      if (view != null) {
        InputMethodManager imm = (InputMethodManager) getSystemService(
            Context.INPUT_METHOD_SERVICE);
        imm.hideSoftInputFromWindow(view.getWindowToken(), 0);
      }
    }
  };

  private IVoIPObserver mCallbacksListener = new IVoIPObserver() {

    @Override
    public void onSipStarted() {

    }

    @Override
    public void onSipStartFailed() {

    }

    @Override
    public void onRegisterState(RegisterState result, int expiry) {

    }

    @Override
    public void onCallState(String callId, CallState callState, String contactNumber) {
      Log.d(TAG, "onCallState: " + callState);
      Intent callActivityIntent;
      switch (callState) {
        case CALL_STATE_RINGING:
          callActivityIntent = new Intent(MainActivity.this, OutgoingCallActivity.class);
          callActivityIntent.putExtra(OutgoingCallActivity.CALL_ID, callId);
          callActivityIntent.putExtra(OutgoingCallActivity.CALL_NUMBER, contactNumber);
          callActivityIntent.setFlags(Intent.FLAG_EXCLUDE_STOPPED_PACKAGES);
          if (mRingingCallId != null && mRingingCallId.equals(callId)) {
            return;
          } else if (mRingingCallId == null) {
            mRingingCallId = callId;
          }
          startActivity(callActivityIntent);
          break;
        case CALL_STATE_CONFIRMED:
        case CALL_STATE_EARLY:
          callActivityIntent = new Intent(MainActivity.this, CallActivity.class);
          callActivityIntent.putExtra("call_id", callId);
          callActivityIntent.putExtra("call_number", contactNumber);
          if (mRingingCallId != null && !mRingingCallId.equals(callId)) {
            return;
          }
          startActivity(callActivityIntent);
          break;
        case CALL_STATE_STARTING:
        case CALL_STATE_CONNECTING:
        case CALL_STATE_CALLING:
          callActivityIntent = new Intent(MainActivity.this, OutgoingCallActivity.class);
          mRingingCallId = callId;
          callActivityIntent.putExtra(OutgoingCallActivity.CALL_ID, callId);
          callActivityIntent.putExtra(OutgoingCallActivity.CALL_NUMBER, contactNumber);
          startActivity(callActivityIntent);
          break;
        default:
          break;
      }
    }

    @Override
    public void onLog(int level, String message) {

    }

    @Override
    public void onSipStopped() {

    }

    @Override
    public void onBluetoothState(BluetoothState bluetoothState) {

    }

    @Override
    public void onAudioRouteChange(AudioRoute route) {

    }

    @Override
    public void onBluetoothBtnState(BluetoothButtonsState state, String command, String eventName,
        Integer buttonID, Integer mode) {

    }

    @Override
    public void onNetworkLost() {

    }

    @Override
    public void onNetworkChanged() {

    }

    @Override
    public void onNetworkRegained() {

    }

    @Override
    public void onLicense(LicensingState state, String description) {

    }

    @Override
    public void onNATTypeDetected(NATType type) {

    }

  };

}