package net.greenfieldtech.cloudonixsdk.cloudonixdialer.activities;

import android.content.Intent;
import android.media.AudioManager;
import android.media.SoundPool;
import android.os.Bundle;
import android.util.Log;
import android.widget.TextView;
import butterknife.Bind;
import butterknife.OnClick;
import java.util.Date;
import net.greenfieldtech.cloudonixsdk.api.interfaces.IVoIPObserver;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.R;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.database.DatabaseHelper;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.models.RecentCall;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils.SdkFacade;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils.VoIPClient;

public class OutgoingCallActivity extends BaseActivity {

  protected static final String TAG = "OutgoingCallActivity";

  public static final String CALL_ID = "call_id";
  public static final String CALL_NUMBER = "call_number";

  @Bind(R.id.tv_current_call_number)
  TextView tvNumber;
  @Bind(R.id.tv_contact_name)
  TextView tvContactName;

  private String mCallId;
  private String mCallNumber = "";

  protected SoundPool mSoundPool;
  protected int mStreamID;

  @Override
  protected int getLayoutResource() {
    return R.layout.activity_outgoing_call;
  }

  @Override
  protected void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);

    // TODO need to remove it ???
    mCallId = getIntent().getStringExtra(CALL_ID);
    mCallNumber = getIntent().getStringExtra(CALL_NUMBER);

    if (SdkFacade.findActiveCall(null) != null) {
      // Already got confirmed before activity was opened
      Log.i(TAG, "onCallConnected");

      mCallId = SdkFacade.findActiveCall(null);
      mCallNumber = VoIPClient.getInstance().getCallUrl(mCallId);

      Intent callIntent = new Intent(this, CallActivity.class);
      callIntent.putExtra(CALL_ID, mCallId);
      callIntent.putExtra(CALL_NUMBER, mCallNumber);
      startActivity(callIntent);

      finish();
    }

    tvNumber.setText(mCallNumber);
    tvContactName.setText("Name - " + mCallNumber);

    // TODO need to remove it ???
    if (mCallId == null) {
      finish();
    }
  }

  @OnClick(R.id.btn_decline)
  void hangup() {
    Log.d(TAG, "Hanging up callId: " + mCallId);
    VoIPClient.getInstance().hangup(mCallId);
  }

  @Override
  public void onBackPressed() {
    Log.d(TAG, "Back key pressed. Hanging up callId: " + mCallId);
    VoIPClient.getInstance().hangup(mCallId);
    super.onBackPressed();
  }

  @Override
  protected void onPause() {
    VoIPClient.getInstance().removeCallbacksListener(callback);
    stopRingingTone();
    super.onPause();
  }

  @Override
  protected void onResume() {
    VoIPClient.getInstance().addCallbacksListener(callback);
    startRingingTone();
    super.onResume();

    if (SdkFacade.findActiveCall(null) != null) {
      mCallId = SdkFacade.findActiveCall(null);
      mCallNumber = VoIPClient.getInstance().getCallUrl(mCallId);

      Intent startCallActivity = new Intent(this, CallActivity.class);
      startCallActivity.putExtra(CALL_ID, mCallId);
      startCallActivity.putExtra(CALL_NUMBER, mCallNumber);
      startActivity(startCallActivity);
      finish();
    }
  }

  private void startRingingTone() {
    mSoundPool = new SoundPool(1, AudioManager.STREAM_SYSTEM, 0);
    int sound = mSoundPool.load(getApplicationContext(), R.raw.ringing, 1);
    mStreamID = mSoundPool.play(sound, 1.0f, 1.0f, 1, -1, 1.0f);
    Log.i(TAG, "startRingingTone, streamID: " + mStreamID);
  }

  private void stopRingingTone() {
    Log.i(TAG, "stopRingingTone");
    if (mSoundPool == null) {
      return;
    }
    if (mStreamID != 0) {
      mSoundPool.stop(mStreamID);
    }
    mSoundPool.release();
  }

  private IVoIPObserver callback = new IVoIPObserver() {

    @Override
    public void onSipStarted() {

    }

    @Override
    public void onSipStartFailed() {

    }

    @Override
    public void onRegisterState(RegisterState result, int expiry) {

    }

    @Override
    public void onCallState(String key, CallState callState, String contactNumber) {
      Log.i(TAG, "onCallState " + callState.toString());
      Intent intent = getIntent();
      Intent callActivityIntent;
      switch (callState) {
        case CALL_STATE_DISCONNECTED:
        case CALL_STATE_DISCONNECTEDDUETOBUSY:
        case CALL_STATE_DISCONNECTEDDUETOTIMEOUT:
        case CALL_STATE_DISCONNECTEDMEDIACHANGED:
        case CALL_STATE_DISCONNECTEDDUETONOMEDIA:
        case CALL_STATE_DISCONNECTEDDUETONETWORKCHANGE:
          Log.i(TAG, "onDisconnected");
          finish();
          break;
        case CALL_STATE_EARLY:
          Log.i(TAG, "onEarlyMedia");
          callActivityIntent = new Intent(OutgoingCallActivity.this, CallActivity.class);
          callActivityIntent.putExtra(CALL_ID, intent.getStringExtra(CALL_ID));
          callActivityIntent.putExtra(CALL_NUMBER, contactNumber);
          startActivity(callActivityIntent);
          finish();
          break;
        case CALL_STATE_MEDIAACTIVE:
        case CALL_STATE_CONFIRMED:
          Log.i(TAG, "onCallConnected");

          DatabaseHelper databaseHelper = DatabaseHelper.getInstance(OutgoingCallActivity.this);
          RecentCall call = new RecentCall("Name - " + contactNumber, contactNumber, new Date());
          call.setIncoming(false);
          databaseHelper.addCall(call);

          callActivityIntent = new Intent(OutgoingCallActivity.this, CallActivity.class);
          callActivityIntent.putExtra(CALL_ID, intent.getStringExtra(CALL_ID));
          callActivityIntent.putExtra(CALL_NUMBER, contactNumber);
          startActivity(callActivityIntent);
          finish();
          break;
        default:
          break;
      }
    }

    @Override
    public void onLog(int level, String message) {

    }

    @Override
    public void onSipStopped() {

    }

    @Override
    public void onBluetoothState(BluetoothState bluetoothState) {

    }

    @Override
    public void onAudioRouteChange(AudioRoute route) {

    }

    @Override
    public void onBluetoothBtnState(BluetoothButtonsState state, String command, String eventName,
        Integer buttonID, Integer mode) {

    }

    @Override
    public void onNetworkLost() {

    }

    @Override
    public void onNetworkChanged() {

    }

    @Override
    public void onNetworkRegained() {

    }

    @Override
    public void onLicense(LicensingState state, String description) {

    }

    @Override
    public void onNATTypeDetected(NATType type) {

    }
  };

}
