package net.greenfieldtech.cloudonixsdk.cloudonixdialer.fragments;

import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.support.annotation.Nullable;
import android.support.design.widget.Snackbar;
import android.text.TextUtils;
import android.util.Log;
import android.util.TypedValue;
import android.view.Gravity;
import android.view.View;
import android.widget.RelativeLayout;
import android.widget.TextView;
import butterknife.Bind;
import butterknife.OnClick;
import butterknife.OnLongClick;
import net.greenfieldtech.cloudonixsdk.api.interfaces.IVoIPObserver;
import net.greenfieldtech.cloudonixsdk.appinterface.CloudonixSDKClient;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.R;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.events.UpdateSettingsEvent;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils.SIPConfig;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils.SdkFacade;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils.VoIPClient;
import net.greenfieldtech.cloudonixsdk.utils.StorageAndPrefsWrapper;
import org.greenrobot.eventbus.EventBus;
import org.greenrobot.eventbus.Subscribe;
import org.greenrobot.eventbus.ThreadMode;

/**
 * Created by alex on 20.04.16.
 */

public class KeypadFragment extends BaseFragment {

  private static final String TAG = "PhoneDialer";
  private static final int NUMBER_LENGTH_LIMIT = 32;
  private static final int UPDATE_REGISTRATION_INTERVAL = 3000;

  @Bind(R.id.root)
  RelativeLayout root;
  @Bind(R.id.tv_number)
  TextView tvNumber;
  @Bind(R.id.tv_registration_status)
  TextView tvRegStatus;

  private String registrationStatus;
  private String number;
  private boolean mIsFirstTime = true;
  private Handler updateRegistrationHandler;

  private float density;
  private int textSizeStep = 0;

  // TODO is it really need?
  private String previousNumber;

  @Override
  public int getLayoutResource() {
    return R.layout.fragment_keypad;
  }

  @Override
  public void onCreate(@Nullable Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);

    density = getResources().getDisplayMetrics().density;

    number = "";
    updateRegistrationHandler = new Handler();
    updateRegistrationHandler.post(updateRegistrationRunnable);
  }

  @Override
  public void onViewCreated(View view, @Nullable Bundle savedInstanceState) {
    super.onViewCreated(view, savedInstanceState);

    VoIPClient.getInstance().setMediaResources(getActivity(), R.raw.busy, R.raw.ringing,
        R.raw.alert, R.raw.end_call, R.raw.hold, R.raw.call_on_hold, R.raw.call_off_hold);

    VoIPClient.getInstance().setLogLevel(CloudonixSDKClient.LOG_LEVEL_ALL);
  }

  @Override
  public void onResume() {
    super.onResume();

    tvNumber.setText(number);
    tvRegStatus.setText(registrationStatus);
    updateRegistrationHandler.postDelayed(updateRegistrationRunnable, UPDATE_REGISTRATION_INTERVAL);

    new Handler().postDelayed(new Runnable() {
      @Override
      public void run() {
        if (getActivity() != null) {
          getActivity().runOnUiThread(mAskPermissionsRunnable);
        }
      }
    }, 1000);
  }

  @Override
  public void onPause() {
    super.onPause();

    number = tvNumber.getText().toString();
    updateRegistrationHandler.removeCallbacks(updateRegistrationRunnable);
  }

  private Runnable updateRegistrationRunnable = new Runnable() {
    @Override
    public void run() {
      updateRegistrationStatus();
      updateRegistrationHandler.postDelayed(this, UPDATE_REGISTRATION_INTERVAL);
    }
  };

  private Runnable mAskPermissionsRunnable = new Runnable() {
    @Override
    public void run() {
      if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
        if (mIsFirstTime) {
          mIsFirstTime = false;
          VoIPClient.getInstance().askForPermissions(getActivity(), true);
        } else {
          VoIPClient.getInstance().askForPermissions(getActivity(), mIsFirstTime);
        }
      }
    }
  };

  private void updateRegistrationStatus() {
    if (isRegistered()) {
      StorageAndPrefsWrapper prefs = StorageAndPrefsWrapper.getInstance();
      String user = prefs.getString(StorageAndPrefsWrapper.PreferencesKeys.ACCOUNT_USERNAME, "");
      registrationStatus = String.format("%s: Registered", user);
    } else {
      onMessageEvent(null); // TODO Ask Arik about it !!!
      registrationStatus = "Not registered";
    }

    if (tvRegStatus != null) {
      tvRegStatus.setText(registrationStatus);
    }
  }

  private boolean isRegistered() {
    if (VoIPClient.getInstance().isRegistered()) {
      return true;
    }
    return false;
  }

  @OnClick(R.id.btn_delete)
  void deleteClicked() {
    if (number.length() > 0) {
      number = number.substring(0, number.length() - 1);
      tvNumber.setText(number);
    }
//
//        float textSize;
//        if (number.length() < 10) {
//            textSize = getResources().getDimension(R.dimen.default_size) / density;
//            textSizeStep --;
//        } else if (number.length() < 16) {
//            textSize = getResources().getDimension(R.dimen.pre_default_text_size) / density;
//            textSizeStep --;
//        } else if (number.length() < 19) {
//            textSize = getResources().getDimension(R.dimen.middle_text_size) / density;
//            textSizeStep --;
//        } else {
//            textSize = getResources().getDimension(R.dimen.min_text_size) / density;
//            textSizeStep --;
//        }
//
//        tvNumber.setTextSize(TypedValue.COMPLEX_UNIT_SP, textSize);
//        tvNumber.setGravity(Gravity.CENTER_VERTICAL | Gravity.CENTER_HORIZONTAL);
    setDefaultTextSize();
  }

  @OnLongClick(R.id.btn_delete)
  boolean deleteNumber() {
    number = "";
    tvNumber.setText(number);
    setDefaultTextSize();
    return true;
  }

  private void setDefaultTextSize() {
    if (TextUtils.isEmpty(tvNumber.getText())) {
      textSizeStep = 0;
      tvNumber.setTextSize(TypedValue.COMPLEX_UNIT_SP,
          getResources().getDimension(R.dimen.default_size) / density);
    }
  }

  @OnClick({R.id.btn_1, R.id.btn_2, R.id.btn_3, R.id.btn_4, R.id.btn_5, R.id.btn_6,
      R.id.btn_7, R.id.btn_8, R.id.btn_9, R.id.btn_0, R.id.btn_star, R.id.btn_pound})
  void onBtnNumberClick(View button) {
    if (tvNumber.getText().length() < NUMBER_LENGTH_LIMIT) {
      try {
        String viewTag = (String) button.getTag();
        try {
          int d = Integer.parseInt(viewTag);
          number = number + d;
          Log.i(TAG, "Number:" + number);
        } catch (NumberFormatException e) {
          if (viewTag.equals("*")) {
            number += "*";
          }
          if (viewTag.equals("#")) {
            number += "#";
          }
        }
        tvNumber.setText(number);
//                customizeTextIfNeed();
      } catch (ClassCastException e) {

      }
    } else {
      Snackbar.make(root, "Limit of number length", Snackbar.LENGTH_SHORT).show();
    }
  }

  private void customizeTextIfNeed() {
    int linesCount = tvNumber.getLineCount();
    float textSize;
    if (linesCount > 1) {
      switch (textSizeStep) {
        case 0: {
          textSize = getResources().getDimension(R.dimen.pre_default_text_size) / density;
          break;
        }
        case 1: {
          textSize = getResources().getDimension(R.dimen.middle_text_size) / density;
          break;
        }
        case 2: {
          textSize = getResources().getDimension(R.dimen.min_text_size) / density;
          break;
        }
        default:
          textSize = getResources().getDimension(R.dimen.min_text_size) / density;
          break;
      }
      tvNumber.setTextSize(TypedValue.COMPLEX_UNIT_SP, textSize);
      textSizeStep++;
      tvNumber.setGravity(Gravity.CENTER_VERTICAL | Gravity.CENTER_HORIZONTAL);
    }
  }

  @OnLongClick(R.id.btn_0)
  boolean addPlusSymbol() {
    number = number + "+";
    tvNumber.setText(number);
    return true;
  }

  @OnClick(R.id.btn_call)
  void doCall() {
    updateRegistrationStatus();
    if (!SdkFacade.isCodecSelected()) {
      Snackbar.make(root, getString(R.string.keypad_error_no_codec), Snackbar.LENGTH_SHORT).show();
      return;
    }
    if (TextUtils.isEmpty(number)) {
      if (previousNumber != null) {
        number = previousNumber;
        tvNumber.setText(previousNumber);
      }
    } else {
      if (isRegistered()) {
        previousNumber = number;
        VoIPClient.getInstance().dial(number);
      } else {
        Snackbar.make(root, getString(R.string.keypad_error_not_registered), Snackbar.LENGTH_SHORT)
            .show();
      }
    }
  }

  @Override
  public void onStart() {
    super.onStart();
    EventBus.getDefault().register(this);
  }

  @Override
  public void onStop() {
    EventBus.getDefault().unregister(this);
    super.onStop();
  }

  @Subscribe(sticky = true, threadMode = ThreadMode.MAIN)
  public void onMessageEvent(UpdateSettingsEvent event) {
    StorageAndPrefsWrapper prefs = StorageAndPrefsWrapper.getInstance();

    boolean ei = prefs.getBoolean(StorageAndPrefsWrapper.PreferencesKeys.ENABLE_ICE, true);
    boolean ur = prefs.getBoolean(StorageAndPrefsWrapper.PreferencesKeys.USE_REGISTRATION, true);
    boolean isOpus = prefs.getBoolean(StorageAndPrefsWrapper.PreferencesKeys.USE_OPUS, true);
    boolean isIlbc = prefs.getBoolean(StorageAndPrefsWrapper.PreferencesKeys.USE_ILBC, true);
    boolean isG722 = prefs.getBoolean(StorageAndPrefsWrapper.PreferencesKeys.USE_G722, true);
    boolean isG711 = prefs.getBoolean(StorageAndPrefsWrapper.PreferencesKeys.USE_G711, true);
    String minAudioRate = prefs.getString(StorageAndPrefsWrapper.PreferencesKeys.MIN_RATE, "8000");
    String maxAudioRate = prefs.getString(StorageAndPrefsWrapper.PreferencesKeys.MAX_RATE, "8000");
    maxAudioRate = "8000";

    SIPConfig conf = SIPConfig.getInstance();
    conf.setEnableIce(ei);
    conf.setUseOpus(isOpus);
    conf.setUseILBC(isIlbc);
    conf.setUseG722(isG722);
    conf.setUseG711(isG711);
    conf.setUseRegistration(ur);
    conf.setMinAudioRate(minAudioRate);
    conf.setMaxAudioRate(maxAudioRate);

    VoIPClient.getInstance().setConfig(IVoIPObserver.ConfigurationKey.ENABLE_ICE, ei ? "1" : "0");
    VoIPClient.getInstance().setConfig(IVoIPObserver.ConfigurationKey.USE_REGISTRATION, ur ? "1" : "0");
    VoIPClient.getInstance().setConfig(IVoIPObserver.ConfigurationKey.USER_AGENT, "Greenfield");
    VoIPClient.getInstance().setConfig(IVoIPObserver.ConfigurationKey.MIN_RATE, "" + minAudioRate);
    VoIPClient.getInstance().setConfig(IVoIPObserver.ConfigurationKey.MAX_RATE, "" + maxAudioRate);
    VoIPClient.getInstance().setConfig(IVoIPObserver.ConfigurationKey.USE_OPUS, isOpus ? "1" : "0");
    VoIPClient.getInstance().setConfig(IVoIPObserver.ConfigurationKey.USE_ILBC, isIlbc ? "1" : "0");
    VoIPClient.getInstance().setConfig(IVoIPObserver.ConfigurationKey.USE_G722, isG722 ? "1" : "0");
    VoIPClient.getInstance().setConfig(IVoIPObserver.ConfigurationKey.USE_G711, isG711 ? "1" : "0");

    VoIPClient.getInstance().setConfig(IVoIPObserver.ConfigurationKey.ENABLE_NAT, "1");
    VoIPClient.getInstance().setConfig(IVoIPObserver.ConfigurationKey.ENABLE_STUN, "1");
    VoIPClient.getInstance().setConfig(IVoIPObserver.ConfigurationKey.STUN_SERVER, "iphone-stun.strato-iphone.de:3478");
    VoIPClient.getInstance().setConfig(IVoIPObserver.ConfigurationKey.DISABLE_SECURE_SIPS, "1");

    String server = prefs.getString(StorageAndPrefsWrapper.PreferencesKeys.ACCOUNT_SERVER, "");
    String username = prefs.getString(StorageAndPrefsWrapper.PreferencesKeys.ACCOUNT_USERNAME, "");
    String password = prefs.getString(StorageAndPrefsWrapper.PreferencesKeys.ACCOUNT_PASSWORD, "");
    String displayName = prefs.getString(StorageAndPrefsWrapper.PreferencesKeys.ACCOUNT_DISPLAYNAME, "");
    String transportType = prefs.getString(StorageAndPrefsWrapper.PreferencesKeys.ACCOUNT_TRANSPORT, "");

    String sPort = prefs.getString(StorageAndPrefsWrapper.PreferencesKeys.ACCOUNT_TRANSPORT_PORT);
    int port = Integer.parseInt((sPort.equals("") ? "5060" : sPort));
    boolean use_dns_srv = prefs.getBoolean(StorageAndPrefsWrapper.PreferencesKeys.ACCOUNT_USE_DNS_SRV, false);
    VoIPClient.getInstance().setAccount(server, username, password, displayName, use_dns_srv, port, transportType);

    if (SettingsFragment.mRequireSdkReboot) {
      Log.d(TAG, "mRequireSdkReboot is: true restarting Voip CLient ");
      VoIPClient.getInstance().restartSip();
      SettingsFragment.mRequireSdkReboot = false;
    }
    EventBus.getDefault().removeStickyEvent(UpdateSettingsEvent.class);
  }

}
