package net.greenfieldtech.cloudonixsdk.cloudonixdialer.fragments;

import android.graphics.Typeface;
import android.os.Bundle;
import android.support.annotation.Nullable;
import android.support.design.widget.Snackbar;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.text.Spannable;
import android.view.View;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;
import butterknife.Bind;
import butterknife.OnClick;
import java.util.ArrayList;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.R;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.adapters.RecentCallsAdapter;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.database.DatabaseHelper;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.models.RecentCall;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils.TypefaceManager;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils.UIUtils;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils.VoIPClient;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.views.DividerItemDecoration;

/**
 * Created by alex on 20.04.16.
 */

public class RecentCallsFragment extends BaseFragment implements RecentCallsAdapter.Callback {

  @Bind(R.id.root)
  LinearLayout root;
  @Bind(R.id.missed_call_container)
  RelativeLayout missedCallsContainer;
  @Bind(R.id.tv_missed_call)
  TextView tvMissedCalls;
  @Bind(R.id.recycler_recent_calls)
  RecyclerView recyclerCalls;
  @Bind(R.id.action_missed_calls)
  TextView tvMissed;
  @Bind(R.id.action_all_calls)
  TextView tvAll;

  private LinearLayoutManager layoutManager;
  private DatabaseHelper databaseHelper;
  private ArrayList<RecentCall> missedCalls;

  private enum CALL {ALL, MISSED}

  @Override
  public int getLayoutResource() {
    return R.layout.fragment_recent_calls;
  }

  @Override
  public void onViewCreated(View view, @Nullable Bundle savedInstanceState) {
    super.onViewCreated(view, savedInstanceState);

    databaseHelper = DatabaseHelper.getInstance(getActivity());

    tvAll.setSelected(true);

    // TODO need to remove it
//        RecentCall call = new RecentCall("Unnamed", "0000", new Date());
//        call.setIncoming(true);
//        call.setMissed(true);
//        call.setViewed(false);
//        databaseHelper.addCall(call);

    if (isMissedCallExist()) {
      showMissedCallsContainer();
    }

    layoutManager = new LinearLayoutManager(getActivity());
    recyclerCalls.setLayoutManager(layoutManager);
    recyclerCalls.addItemDecoration(
        new DividerItemDecoration(getActivity(), DividerItemDecoration.LIST_VERTICAL));

    loadCallsList(CALL.ALL);
  }

  private boolean isMissedCallExist() {
    missedCalls = new ArrayList<>(databaseHelper.getNotViewedMissedCalls());
    if (missedCalls != null && !missedCalls.isEmpty()) {
      return true;
    }
    return false;
  }

  private void showMissedCallsContainer() {
    missedCallsContainer.setVisibility(View.VISIBLE);
    String message;
    int missedCallsCount = missedCalls.size();
    if (missedCallsCount > 1) {
      message = "You have missed %s calls";
    } else {
      message = "You have missed %s call";
    }
    String formattedMessage = String.format(message, missedCallsCount);
    tvMissedCalls.setText(highLighCallsNumber(formattedMessage));
  }

  private Spannable highLighCallsNumber(String message) {
    int start = message.indexOf("missed") + 6;
    int end = message.lastIndexOf("call");
    Typeface mTitleTypeface = TypefaceManager
        .obtainTypeface(getActivity(), TypefaceManager.OPEN_SANS_SEMI_BOLD);
    return UIUtils
        .getTypefaceSpannable(getActivity(), message, 19, start, end, R.color.blue_text_color,
            mTitleTypeface);
  }

  void loadCallsList(CALL type) {
    ArrayList<RecentCall> calls;
    if (type == CALL.ALL) {
      calls = new ArrayList<>(databaseHelper.getAllCalls());
    } else {
      calls = new ArrayList<>(databaseHelper.getMissedCalls());
    }
    RecentCallsAdapter adapter = new RecentCallsAdapter(getActivity(), calls);
    adapter.setCallback(this);
    recyclerCalls.setAdapter(adapter);
  }

  @OnClick(R.id.action_all_calls)
  void showAllCalls() {
    loadCallsList(CALL.ALL);

    tvAll.setSelected(true);
    tvMissed.setSelected(false);
  }

  @OnClick(R.id.action_missed_calls)
  void showMissedCalls() {
    loadCallsList(CALL.MISSED);

    tvAll.setSelected(false);
    tvMissed.setSelected(true);
  }

  @OnClick(R.id.iv_missed_call)
  void hideMissedCallsView() {
    missedCallsContainer.setVisibility(View.GONE);

    markAllMissedAsViewed();
  }

  private void markAllMissedAsViewed() {
    if (missedCalls != null) {
      for (RecentCall call : missedCalls) {
        call.setViewed(true);
        databaseHelper.updateCall(call);
      }
    }
  }

  @Override
  public void onCallClick(String contactNumber) {
    VoIPClient client = VoIPClient.getInstance();
    if (client.isRegistered()) {
      client.dial(contactNumber);
    } else {
      Snackbar.make(root, getString(R.string.activity_call_not_registered), Snackbar.LENGTH_SHORT)
          .show();
    }
  }

}
