package net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils;

import android.content.Context;
import android.content.SharedPreferences;
import android.os.Build;
import android.preference.PreferenceManager;
import net.greenfieldtech.cloudonixsdk.api.interfaces.IVoIPObserver;

/**
 * Created by arikhalperin on 12/11/2015.
 */
public class SIPConfig {

  static SIPConfig sInstance = null;

  SharedPreferences mSharedPrefs;

  private SIPConfig(Context context) {
    mSharedPrefs = PreferenceManager.getDefaultSharedPreferences(context);
  }


  public static void create(Context context) {
    sInstance = new SIPConfig(context);
  }

  public static SIPConfig getInstance() {
    return sInstance;
  }

  public boolean getEnableIce() {
    return mSharedPrefs
        .getBoolean(IVoIPObserver.ConfigurationKey.ENABLE_ICE.toString().toLowerCase(), true);
  }

  public void setEnableIce(boolean enableIce) {
    SharedPreferences.Editor ed = mSharedPrefs.edit();
    ed.putBoolean(IVoIPObserver.ConfigurationKey.ENABLE_ICE.toString().toLowerCase(), enableIce);
    saveChanges(ed);
  }

  public boolean getUseRegisteration() {
    return mSharedPrefs
        .getBoolean(IVoIPObserver.ConfigurationKey.USE_REGISTRATION.toString().toLowerCase(), true);
  }

  public void setUseRegistration(boolean useRegistration) {
    SharedPreferences.Editor ed = mSharedPrefs.edit();
    ed.putBoolean(IVoIPObserver.ConfigurationKey.USE_REGISTRATION.toString().toLowerCase(),
        useRegistration);
    saveChanges(ed);
  }

  public String getUserAgent() {
    return mSharedPrefs
        .getString(IVoIPObserver.ConfigurationKey.USER_AGENT.toString().toLowerCase(),
            "CloudonixSDK");
  }

  public void setUserAgent(String userAgent) {
    SharedPreferences.Editor ed = mSharedPrefs.edit();
    ed.putString(IVoIPObserver.ConfigurationKey.USER_AGENT.toString().toLowerCase(), userAgent);
    saveChanges(ed);
  }

  public boolean getUseOpus() {
    return mSharedPrefs
        .getBoolean(IVoIPObserver.ConfigurationKey.USE_OPUS.toString().toLowerCase(), true);
  }

  public void setUseOpus(boolean useOpus) {
    SharedPreferences.Editor ed = mSharedPrefs.edit();
    ed.putBoolean(IVoIPObserver.ConfigurationKey.USE_OPUS.toString().toLowerCase(), useOpus);
    saveChanges(ed);
  }

  public boolean getUseG711() {
    return mSharedPrefs
        .getBoolean(IVoIPObserver.ConfigurationKey.USE_G711.toString().toLowerCase(), false);
  }

  public void setUseG711(boolean useG711) {
    SharedPreferences.Editor ed = mSharedPrefs.edit();
    ed.putBoolean(IVoIPObserver.ConfigurationKey.USE_G711.toString().toLowerCase(), useG711);
    saveChanges(ed);
  }


  public boolean getUseG722() {
    return mSharedPrefs
        .getBoolean(IVoIPObserver.ConfigurationKey.USE_G722.toString().toLowerCase(), false);
  }

  public void setUseG722(boolean useG722) {
    SharedPreferences.Editor ed = mSharedPrefs.edit();
    ed.putBoolean(IVoIPObserver.ConfigurationKey.USE_G722.toString().toLowerCase(), useG722);
    saveChanges(ed);
  }

  public boolean getUseILBC() {
    return mSharedPrefs
        .getBoolean(IVoIPObserver.ConfigurationKey.USE_ILBC.toString().toLowerCase(), false);
  }

  public void setUseILBC(boolean useILBC) {
    SharedPreferences.Editor ed = mSharedPrefs.edit();
    ed.putBoolean(IVoIPObserver.ConfigurationKey.USE_ILBC.toString().toLowerCase(), useILBC);
    saveChanges(ed);
  }


  public String getMinAudioRate() {
    return mSharedPrefs
        .getString(IVoIPObserver.ConfigurationKey.MIN_RATE.toString().toLowerCase(), "8000");
  }

  public void setMinAudioRate(String rate) {
    SharedPreferences.Editor ed = mSharedPrefs.edit();
    ed.putString(IVoIPObserver.ConfigurationKey.MIN_RATE.toString().toLowerCase(), rate);
    saveChanges(ed);
  }

  public String getMaxAudioRate() {
    return mSharedPrefs
        .getString(IVoIPObserver.ConfigurationKey.MAX_RATE.toString().toLowerCase(), "24000");
  }

  public void setMaxAudioRate(String rate) {
    SharedPreferences.Editor ed = mSharedPrefs.edit();
    ed.putString(IVoIPObserver.ConfigurationKey.MAX_RATE.toString().toLowerCase(), rate);
    saveChanges(ed);
  }

  private void saveChanges(SharedPreferences.Editor editor) {
    if (Build.VERSION.SDK_INT > 10) {
      editor.apply();
    } else {
      editor.commit();
    }
  }

}
